package fr.username404.snowygui.gui.feature

import fr.username404.snowygui.ClickGui
import fr.username404.snowygui.gui.elements.ClickBox
import net.minecraft.network.chat.Component.translatable
import net.minecraft.network.chat.contents.TranslatableContents

/**
 * Should be used on classes/objects extending [ButtonImpl]
 */
@Target(AnnotationTarget.CLASS)
@Retention(AnnotationRetention.RUNTIME)
annotation class ButtonInfo(
    val parent: Category,
    val kind: Type = Type.TOGGLE,
    @Deprecated("Use shouldSave() instead", level = DeprecationLevel.ERROR)
    val shouldSave: Boolean = true, /** Whether or not the state of the button should be saved */
    val ignored: Boolean = false /** Excludes a class from button collection */
) {
    companion object {
        const val lightningFactor: Float = 0.33F
        enum class Type {
            TOGGLE,
            CLICK
        }
        enum class Category(translationKey: String, val categoryColor: Int, val shouldHide: Boolean = false) {
            MISC("snowy.clickbox.misc", Colors.BLUE),
            HUD("snowy.clickbox.hud", Colors.DARK_PURPLE),
            MACROS("snowy.clickbox.macros", Colors.GREEN);
            companion object {
                fun fromBox(box: ClickBox): Category? = values().find {
                    box.isCategory(it)
                }
            }
            var box: ClickBox = ClickBox(
                x = 4.0 + (ordinal - ClickGui.clickBoxes.count { category ->
                    category.hidden
                }) * 86, y = 4.0,
                name = translatable(translationKey),
                color = categoryColor
            ); private set
            private fun addBox() = ClickGui.components.add(box.apply { if (shouldHide) hidden = true })
            constructor(
                translationKey: String, categoryColor: Colors, condition: Boolean = false
            ): this(translationKey, categoryColor.hexValue, condition)
            constructor(categoryBox: ClickBox): this(translationKey = (categoryBox.name.contents as TranslatableContents).key, categoryColor = categoryBox.color) {
                with(ClickGui.components) {
                    remove(this@Category.box)
                    this@Category.box = categoryBox
                    addBox()
                }
            }
            init { addBox() }
        }
    }
}

fun ButtonInfo.shouldSave(): Boolean = @Suppress("DEPRECATION_ERROR") shouldSave && kind == ButtonInfo.Companion.Type.TOGGLE

typealias Category = ButtonInfo.Companion.Category